/*
This file is part of IRCaBot.
IRCaBot is IRC logger with features.
Source code: https://notabug.org/acetone/ircabot.
Copyright (C) 2023, acetone.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "currenttime.h"
#include "adminircpanel.h"
#include "ircnetworkclient.h"
#include "config.h"

#include <QMutex>
#include <QRegularExpression>

QMap<QString,qint64> AdminIRCPanel::m_identifiedTimestamps;

void AdminIRCPanel::identifiedTimestampUpdate(const QString& networkName)
{
    m_identifiedTimestamps[networkName] = currentTimestampSecs();
}

bool AdminIRCPanel::parse(const QString& networkName, const QString &input, QStringList& answer)
{
    QMutex mtx;
    if (not mtx.tryLock())
    {
        answer.push_back("The admin panel is busy. Try later.");
        return true;
    }
    mtx.unlock();
    QMutexLocker lock (&mtx);

    if (not m_identifiedTimestamps.contains(networkName) or
        currentTimestampSecs() - m_identifiedTimestamps[networkName] > Config::ircAdminAuthorizationTimeout())
    {
        answer.push_back("Your authorization is being verified. If your nickname is not registered, chat control is not possible.");
        return false;
    }

    static const QRegularExpression rgx_help          ("^help$", QRegularExpression::CaseInsensitiveOption);
    static const QRegularExpression rgx_channel       ("^channel (add|delete) [^\\s]{1,100}$", QRegularExpression::CaseInsensitiveOption);
    static const QRegularExpression rgx_voiceGate     ("^voiceGate (enable|disable)$", QRegularExpression::CaseInsensitiveOption);
    static const QRegularExpression rgx_reloadChannels("^reload channels$", QRegularExpression::CaseInsensitiveOption);
    static const QRegularExpression rgx_updateCfg     ("^update config$", QRegularExpression::CaseInsensitiveOption);

    if (input.contains(rgx_help))
    {
        answer = QString("Add channel to my list or remove one: channel <add|delete> CHANNEL_NAME,"
                         "After deleting or adding channels, reload connections: reload channels,"
                         "Captcha gate in channels where I have the privilege to do +v: voiceGate <enable|disable>,"
                         "Overwrite the configuration file with the current data: update config").split(',');
    }

    else if (input.contains(rgx_channel))
    {
        bool add = input.contains("channel add", Qt::CaseInsensitive);
        QString channel = input;
        static QRegularExpression rgx_channel("^channel (add|delete) ", QRegularExpression::CaseInsensitiveOption);
        channel.remove(rgx_channel);

        auto configs = Config::networksAccess();
        for (auto iter = configs.begin(); iter != configs.end(); ++iter)
        {
            auto& network = *iter;
            if (network.networkName != networkName) continue;

            if (add and network.channels.contains(channel))
            {
                answer.push_back("Channel " + channel + " already exists.");
                return true;
            }
            else if (add)
            {
                network.channels.push_back(channel);
                answer.push_back("Success. Channel " + channel + " added.");
                return true;
            }

            else if (not add and not network.channels.contains(channel))
            {
                answer.push_back("Channel " + channel + " not exists.");
                return true;
            }
            else if (not add)
            {
                network.channels.removeOne(channel);
                answer.push_back("Success. Channel " + channel + " removed.");
                return true;
            }
        }
        answer.push_back("Failed. Network " + networkName + " not found. Seems like a bug!");
    }

    else if (input.contains(rgx_reloadChannels))
    {
        auto configs = Config::networksAccess();
        for (auto iter = configs.begin(); iter != configs.end(); ++iter)
        {
            iter->instance->reloadChannelList();
        }
        answer.push_back("Success. Reloaded.");
    }

    else if (input.contains(rgx_voiceGate))
    {
        bool enable = input.endsWith("enable", Qt::CaseInsensitive);
        auto configs = Config::networksAccess();
        for (auto iter = configs.begin(); iter != configs.end(); ++iter)
        {
            auto& network = *iter;
            if (network.networkName != networkName) continue;

            if (network.voiceGate == enable)
            {
                answer.push_back("Voice gate already " + QString(enable ? "enabled" : "disabled") + ".");
                return true;
            }
            network.voiceGate = enable;
            // TODO: update ircnetworkclient instance
            answer.push_back("Success. Voice gate " + QString(enable ? "enabled" : "disabled") + ".");
            return true;
        }
        answer.push_back("Failed. Network " + networkName + " not found. Seems like a bug!");
    }

    else if (input.contains(rgx_updateCfg))
    {
        QString error;
        bool status = Config::updateConfigWithCurrentData();
        answer.push_back(status ? "Success. Configuration file updated." : "Failure: " + error + ".");
    }

    else
    {
        answer.push_back("Unknown command. Try \"help\" :)");
    }

    return true;
}
